/*
 *  Copyright (C) 2002 Steve Harris, ECS, University of Southampton
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 */

#ifndef RDFSQL_H
#define RDFSQL_H

#include <glib.h>

#include "rdfsql_datatypes.h"
#include "rdql_datatypes.h"
#include "rdql.h"

/* Functions */

/* rs_hash: the hashing function, returns a 64bit unsigned int that represents
 * the string s, passed in.
 *
 * The current implementation uses the top 64 bits of the MD5 hash of the
 * string, but that should not be depeneded upon. */

guint64 rs_hash(const char *);

/* rs_connect: opens a connection to a local database, if model is NULL it will
 * open with the system default model */

RDFSQL *rs_connect(const char *db, const char *user, const char *passwd, const
		char *model);

/* rs_connect_remote: opens a connection to a remote database, host is a domain
 * name or IP address, eg "triplestore.aktors.org". If model is NULL it will
 * open with the system default model */

RDFSQL *rs_connect_remote(const char *host, const char *db, const char *user, 
                          const char *passwd, const char *model);

void rs_disconnect(RDFSQL *rs);

/* rs_check_model: verifies that a given model URI is in the models table */

guint64 rs_check_model(RDFSQL *this, const char *model);

/* rs_flush: flushes the triples from the model specified in the call to
 * rs_connect. */

void rs_flush(RDFSQL *this);

/* rs_flush_other: flushes triples from the given model */

void rs_flush_other(RDFSQL *this, guint64 model);

/* rs_import: load an RDF file from the web or local disk into the store */
int rs_import(RDFSQL *db, char *uri, char *model, int verbosity);

/* rs_update_caches: check to see if the taxonomic caches need updating and
 * update them if they do. should be called after rs_import() calls */
void rs_update_caches(RDFSQL *db, int verbosity);

/* rs_assert_triple: assert a triple into the KB, type must be one of ObjURI
 * or ObjLiteral
 */
int rs_assert_triple(const RDFSQL *this, const char *subj, const char *pred,
		const char *obj, const rs_obj_type type);

/* rs_assert_inferred_triple: assert a triple into the KB which will be marked
 * as inferred, type must be one of ObjURI or ObjLiteral
 */
int rs_assert_inferred_triple(const RDFSQL *this, const char *subj, const char
		*pred, const char *obj, const rs_obj_type type);

/* rs_assert_raw: asserts a triple given its hashed subject, predicate and
 * object components. Has no safty checking of URIs or Literals, so only use it
 * if you know for sure that the URIs and Literals have allready been asserted.
 */
int rs_assert_raw(const RDFSQL *this, guint64 subj, guint64 pred, guint64 obj,
                  const rs_obj_type type, int inferred);

/* rs_assert_raw_block: similar to asert raw, but asserts a vector of triples
 * in one go. Used to get more poerformance in bulk update tools. */

int rs_assert_raw_block(const RDFSQL *this, rs_raw_triple *block, int count,
                  const rs_obj_type type, int inferred);

/* rs_optimize_tables: causes the underlying SQL representation to optimize its
 * internal caches and indexes. May take a minuite or so, its recommended that
 * you call this function roughly every 24 hours, eg. in a cron job. */

void rs_optimize_tables(RDFSQL *this, int verbosity);

/* rs_find_triples: returns a pointer to a result object containing
 * the triples matching the pattern, possibly including inferred
 * triples.
 *
 * subj, pred and ojb cn be wither strings of the fully qualified URI
 * or NULL, in which case any URI/Literal will match for that part.
 *
 * type may be ObjURI, ObjLiteral or ObjAny. ObjAny is the slowest.
 *
 * num_triples can be a limit on the number of returned triples, or 0, for no
 * limit. Limiting the maximum number will improve performance.
 *
 * Example of use (find all the instances of 'class'):
 *
 *     rs_triple *instances =
 *      rs_find_triples(okbc_intl_db, NULL, RDF_TYPE, class, ObjURI, 0, NULL);
 *
 *     while (instances && (t = rs_next_triple(instances))) {
 *         printf("%s\n", t->subject);
 *     }
 *     rs_free_result(instances);
 *
 * NB Once rs_find_triples has been called, all the triples /must/ be fetched
 * with rs_next_triple(), even if they are not required.
 */

rs_result *rs_find_triples(RDFSQL *this, const char *subj, const char *pred,
		const char *obj, const rs_obj_type type, int num_triples,
		const char *model);

/* rs_find_asserted_triples: returns a pointer to a result object
 * containing only those triples matching the pattern that were
 * directly asserted (ie. not inferred)
 *
 * In all other respects, this function is identical to rs_find_triples.
 */

rs_result *rs_find_asserted_triples(RDFSQL *this, const char *subj, const char *pred,
                const char *obj, const rs_obj_type type, int num_triples,
                const char *model);
/* rs_num_triples: returns the number of triples ina result set */

int rs_num_triples(rs_result *res);

/* rs_triple: return the next triple form a result set. The triple object will
 * have the subject, predicate and object members filled out */

rs_triple *rs_next_triple(rs_result *res);

/* rs_free_results: frees up the memeory used by the results set object */

void rs_free_result(rs_result *res);

/* rs_rebuild_taxonomy: causes the internal caches to be rebuilt to reflect the
 * stored schemata */

void rs_rebuild_taxonomy(RDFSQL *db, int verbosity);

/* rs_entail_rdfs2: generate the entailments for rdfs2 in the given model */

void rs_entail_rdfs2(RDFSQL *db, guint64 model);

/* rs_entail_rdfs3: generate the entailments for rdfs2 in the given model */

void rs_entail_rdfs3(RDFSQL *db, guint64 model);


rs_result *rs_find_all_resources(const RDFSQL *this, int limit, const char
		*model);

rs_result *rs_find_all_models(const RDFSQL *this);

#endif
